==> cordova-plugin-swt-x9-2.0.1.tgz: package/www/X9.js <==
var SERVICE_USER_DATA = '1817';
var CHAR_START_HEART_RATE_MONITORING = '2a5f';
var CHAR_READ_HEART_RATE_DATA = '6e400007-b5a3-f393-e0a9-e50e24dcca9e';
var CHAR_READ_SLEEP_DATA = '6e400006-b5a3-f393-e0a9-e50e24dcca9e';

var START_HEART_RATE_MONITORING = 0x31;

var verbose = false;
var scanning = false;
var SCANNED_X9S = [];
var CONNECTED_X9_MAC;

// Perform one scan
// Returns a single promise
exports.scan = function() {
  return new Promise(function (resolve, reject) {
    if (verbose)
      console.log('cordova-plugin-swt-x9: scan started');
    scanning = true;

    ble.startScan([], function(device) {
      if (device['name'] == 'X9') {
        if (verbose)
          console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
        if (SCANNED_X9S.indexOf(device['id']) < 0) { // New device found
          SCANNED_X9S.push(device['id']);
          ble.stopScan();
          scanning = false;
          resolve(device['id']);
        }
      }
    }, function(error) {
      if (verbose)
        console.error('cordova-plugin-swt-x9: scan error', error);
      scanning = false;
      reject(error);
    });
  });
}

exports.stopScan = function() {
  return new Promise(function (resolve, reject) {
    ble.stopScan(function(result) {
      if (verbose)
        console.log('cordova-plugin-swt-x9: stopped scan');
      resolve(result);
    }, function(error) {
      if (verbose)
        console.error('cordova-plugin-swt-x9: stopScan error');
      reject(error);
    });
  });
}

// Get X9 heart rate data
// If not connected and if no specific MAC found, it will start a scan, connect, etc.
exports.readHeartRateData = function(MAC = CONNECTED_X9_MAC) {
  return new Promise(function (resolve, reject) {
    if (!MAC && !CONNECTED_X9_MAC) {
      // No MAC provided, no connected X9 MAC found, need to start scan, connect, etc...
      // TODO
    } else {
      ble.isConnected(MAC, function() {
        // Connected to X9.
        if (verbose)
          console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (1)');
        var dataStartHeartMonitoring = new Uint8Array(1);
        dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
        ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
          if (verbose)
            console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (1)');
          ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications... (1)');
            var responseUintArray = new Uint8Array(response2);
            var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
            resolve(responseUintArray, responseStr);
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: heart rate notification error (1)');
            reject(error);
          });
        }, function(error) {
          if (verbose)
            console.error('cordova-plugin-swt-x9: write error (1)');
          reject(error);
        });
      }, function() {
        // X9 not connected, need to connect...
        if (verbose)
          console.log('cordova-plugin-swt-x9: connecting to X9...');
        if (SCANNED_X9S.indexOf(MAC) > -1) {
          // Specified X9 MAC has been previously scanned, connect
          ble.connect(MAC, function(device) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected to X9');
            if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
              CONNECTED_X9_MAC = MAC;
            }

            if (verbose)
              console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (2)');
            var dataStartHeartMonitoring = new Uint8Array(1);
            dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
            ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
              if (verbose)
                console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (2)');
              ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
                if (verbose)
                  console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications...');
                var responseUintArray = new Uint8Array(response2);
                var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
                resolve(responseUintArray, responseStr);
              }, function(error) {
                if (verbose)
                  console.error('cordova-plugin-swt-x9: heart rate notification error (2)');
                reject(error);
              });
            }, function(error) {
              if (verbose)
                console.error('cordova-plugin-swt-x9: write error (2)');
              reject(error);
            });
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: connection error (2)');
            reject(error);
          });
        } else {
          // Specified X9 MAC has not been scanned previously, scan for it and connect if found
          ble.startScan([], function(device) {
            if (device['name'] == 'X9') {
              if (verbose)
                console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
              if (device['id'] == MAC) { // X9 found...
                ble.connect(MAC, function(device) {
                  if (verbose)
                    console.log('cordova-plugin-swt-x9: connected to X9');
                  if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
                    CONNECTED_X9_MAC = MAC;
                  }

                  if (verbose)
                    console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (3)');
                  var dataStartHeartMonitoring = new Uint8Array(1);
                  dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
                  ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
                    if (verbose)
                      console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (3)');
                    ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
                      if (verbose)
                        console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications...');
                      var responseUintArray = new Uint8Array(response2);
                      var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
                      resolve(responseUintArray, responseStr);
                    }, function(error) {
                      if (verbose)
                        console.error('cordova-plugin-swt-x9: heart rate notification error (3)');
                      reject(error);
                    });
                  }, function(error) {
                    if (verbose)
                      console.error('cordova-plugin-swt-x9: write error (3)');
                    reject(error);
                  });
                }, function(error) {
                  if (verbose)
                    console.error('cordova-plugin-swt-x9: connection error (3)');
                  reject(error);
                });
              }
            }
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: scan error', error);
            scanning = false;
            reject(error);
          });

          // TODO: timeout
        }
      });
    }
  });
}

// Get X9 sleep data
// If not connected and if no specific MAC found, it will start a scan, connect, etc.
exports.readSleepData = function(MAC = CONNECTED_X9_MAC) {
  return new Promise(function (resolve, reject) {
    if (!MAC && !CONNECTED_X9_MAC) {
      // No MAC provided, no connected X9 MAC found, need to start scan, connect, etc...
      // TODO
    } else {
      ble.isConnected(MAC, function() {
        // Connected to X9.
        if (verbose)
          console.log('cordova-plugin-swt-x9: listening for sleep data notification response (1)');
        ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
          if (verbose)
            console.log('cordova-plugin-swt-x9: sleep data notification response (1)', response);
          var responseUintArray = new Uint8Array(response);
          var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
          resolve(responseUintArray, responseStr);
        }, function(error) {
          if (verbose)
            console.error('cordova-plugin-swt-x9: sleep notification error (1)');
          reject(error);
        });
      }, function() {
        // X9 not connected, need to connect...
        if (verbose)
          console.log('cordova-plugin-swt-x9: connecting to X9...');
        if (SCANNED_X9S.indexOf(MAC) > -1) {
          // Specified X9 MAC has been previously scanned, connect
          ble.connect(MAC, function(device) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected to X9');
            if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
              CONNECTED_X9_MAC = MAC;
            }

            if (verbose)
              console.log('cordova-plugin-swt-x9: listening for sleep data notification response (2)');
            ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
              if (verbose)
                console.log('cordova-plugin-swt-x9: sleep data notification response (2)', response);
              var responseUintArray = new Uint8Array(response);
              var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
              resolve(responseUintArray, responseStr);
            }, function(error) {
              if (verbose)
                console.error('cordova-plugin-swt-x9: sleep notification error (2)');
              reject(error);
            });
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: connection error (2)');
            reject(error);
          });
        } else {
          // Specified X9 MAC has not been scanned previously, scan for it and connect if found
          ble.startScan([], function(device) {
            if (device['name'] == 'X9') {
              if (verbose)
                console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
              if (device['id'] == MAC) { // X9 found...
                ble.connect(MAC, function(device) {
                  if (verbose)
                    console.log('cordova-plugin-swt-x9: connected to X9');
                  if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
                    CONNECTED_X9_MAC = MAC;
                  }

                  if (verbose)
                    console.log('cordova-plugin-swt-x9: listening for sleep data notification response (3)');
                  ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
                    if (verbose)
                      console.log('cordova-plugin-swt-x9: sleep data notification response (3)', response);
                    var responseUintArray = new Uint8Array(response);
                    var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
                    resolve(responseUintArray, responseStr);
                  }, function(error) {
                    if (verbose)
                      console.error('cordova-plugin-swt-x9: sleep notification error (3)');
                    reject(error);
                  });
                }, function(error) {
                  if (verbose)
                    console.error('cordova-plugin-swt-x9: connection error (3)');
                  reject(error);
                });
              }
            }
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: scan error', error);
            scanning = false;
            reject(error);
          });

          // TODO: timeout
        }
      });
    }
  });
}

exports.getVerbosity = function() {
  return verbose;
}

exports.toggleVerbosity = function() { // toggle verbose console logging mode
  verbose = !verbose;
  if (verbose)
    console.log('cordova-plugin-swt-x9: toggled debug mode, set to', verbose);
}

==> cordova-plugin-swt-x9-2.0.2.tgz: package/www/X9.js <==
var SERVICE_USER_DATA = '1817';
var CHAR_START_HEART_RATE_MONITORING = '2a5f';
var CHAR_READ_HEART_RATE_DATA = '6e400007-b5a3-f393-e0a9-e50e24dcca9e';
var CHAR_READ_SLEEP_DATA = '6e400006-b5a3-f393-e0a9-e50e24dcca9e';

var START_HEART_RATE_MONITORING = 0x31;

var verbose = false;
var scanning = false;
var SCANNED_X9S = [];
var CONNECTED_X9_MAC;

// Perform one scan
// Returns a single promise
exports.scan = function() {
  return new Promise(function (resolve, reject) {
    if (verbose)
      console.log('cordova-plugin-swt-x9: scan started');
    scanning = true;

    ble.startScan([], function(device) {
      if (device['name'] == 'X9') {
        if (verbose)
          console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
        if (SCANNED_X9S.indexOf(device['id']) < 0) { // New device found
          SCANNED_X9S.push(device['id']);
          ble.stopScan();
          scanning = false;
          resolve(device['id']);
        }
      }
    }, function(error) {
      if (verbose)
        console.error('cordova-plugin-swt-x9: scan error', error);
      scanning = false;
      reject(error);
    });
  });
}

exports.stopScan = function() {
  return new Promise(function (resolve, reject) {
    ble.stopScan(function(result) {
      if (verbose)
        console.log('cordova-plugin-swt-x9: stopped scan');
      resolve(result);
    }, function(error) {
      if (verbose)
        console.error('cordova-plugin-swt-x9: stopScan error');
      reject(error);
    });
  });
}

// Get X9 heart rate data
// If not connected and if no specific MAC found, it will start a scan, connect, etc.
exports.readHeartRateData = function(MAC) {
  return new Promise(function (resolve, reject) {
    if (!MAC)
      MAC = CONNECTED_X9_MAC
    
    if (!MAC && !CONNECTED_X9_MAC) {
      // No MAC provided, no connected X9 MAC found, need to start scan, connect, etc...
      // TODO
    } else {
      ble.isConnected(MAC, function() {
        // Connected to X9.
        if (verbose)
          console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (1)');
        var dataStartHeartMonitoring = new Uint8Array(1);
        dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
        ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
          if (verbose)
            console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (1)');
          ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications... (1)');
            var responseUintArray = new Uint8Array(response2);
            var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
            resolve(responseUintArray, responseStr);
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: heart rate notification error (1)');
            reject(error);
          });
        }, function(error) {
          if (verbose)
            console.error('cordova-plugin-swt-x9: write error (1)');
          reject(error);
        });
      }, function() {
        // X9 not connected, need to connect...
        if (verbose)
          console.log('cordova-plugin-swt-x9: connecting to X9...');
        if (SCANNED_X9S.indexOf(MAC) > -1) {
          // Specified X9 MAC has been previously scanned, connect
          ble.connect(MAC, function(device) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected to X9');
            if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
              CONNECTED_X9_MAC = MAC;
            }

            if (verbose)
              console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (2)');
            var dataStartHeartMonitoring = new Uint8Array(1);
            dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
            ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
              if (verbose)
                console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (2)');
              ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
                if (verbose)
                  console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications...');
                var responseUintArray = new Uint8Array(response2);
                var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
                resolve(responseUintArray, responseStr);
              }, function(error) {
                if (verbose)
                  console.error('cordova-plugin-swt-x9: heart rate notification error (2)');
                reject(error);
              });
            }, function(error) {
              if (verbose)
                console.error('cordova-plugin-swt-x9: write error (2)');
              reject(error);
            });
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: connection error (2)');
            reject(error);
          });
        } else {
          // Specified X9 MAC has not been scanned previously, scan for it and connect if found
          ble.startScan([], function(device) {
            if (device['name'] == 'X9') {
              if (verbose)
                console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
              if (device['id'] == MAC) { // X9 found...
                ble.connect(MAC, function(device) {
                  if (verbose)
                    console.log('cordova-plugin-swt-x9: connected to X9');
                  if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
                    CONNECTED_X9_MAC = MAC;
                  }

                  if (verbose)
                    console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (3)');
                  var dataStartHeartMonitoring = new Uint8Array(1);
                  dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
                  ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
                    if (verbose)
                      console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (3)');
                    ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
                      if (verbose)
                        console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications...');
                      var responseUintArray = new Uint8Array(response2);
                      var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
                      resolve(responseUintArray, responseStr);
                    }, function(error) {
                      if (verbose)
                        console.error('cordova-plugin-swt-x9: heart rate notification error (3)');
                      reject(error);
                    });
                  }, function(error) {
                    if (verbose)
                      console.error('cordova-plugin-swt-x9: write error (3)');
                    reject(error);
                  });
                }, function(error) {
                  if (verbose)
                    console.error('cordova-plugin-swt-x9: connection error (3)');
                  reject(error);
                });
              }
            }
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: scan error', error);
            scanning = false;
            reject(error);
          });

          // TODO: timeout
        }
      });
    }
  });
}

// Get X9 sleep data
// If not connected and if no specific MAC found, it will start a scan, connect, etc.
exports.readSleepData = function(MAC) {
  return new Promise(function (resolve, reject) {
    if (!MAC)
      MAC = CONNECTED_X9_MAC
    
    if (!MAC && !CONNECTED_X9_MAC) {
      // No MAC provided, no connected X9 MAC found, need to start scan, connect, etc...
      // TODO
    } else {
      ble.isConnected(MAC, function() {
        // Connected to X9.
        if (verbose)
          console.log('cordova-plugin-swt-x9: listening for sleep data notification response (1)');
        ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
          if (verbose)
            console.log('cordova-plugin-swt-x9: sleep data notification response (1)', response);
          var responseUintArray = new Uint8Array(response);
          var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
          resolve(responseUintArray, responseStr);
        }, function(error) {
          if (verbose)
            console.error('cordova-plugin-swt-x9: sleep notification error (1)');
          reject(error);
        });
      }, function() {
        // X9 not connected, need to connect...
        if (verbose)
          console.log('cordova-plugin-swt-x9: connecting to X9...');
        if (SCANNED_X9S.indexOf(MAC) > -1) {
          // Specified X9 MAC has been previously scanned, connect
          ble.connect(MAC, function(device) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected to X9');
            if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
              CONNECTED_X9_MAC = MAC;
            }

            if (verbose)
              console.log('cordova-plugin-swt-x9: listening for sleep data notification response (2)');
            ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
              if (verbose)
                console.log('cordova-plugin-swt-x9: sleep data notification response (2)', response);
              var responseUintArray = new Uint8Array(response);
              var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
              resolve(responseUintArray, responseStr);
            }, function(error) {
              if (verbose)
                console.error('cordova-plugin-swt-x9: sleep notification error (2)');
              reject(error);
            });
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: connection error (2)');
            reject(error);
          });
        } else {
          // Specified X9 MAC has not been scanned previously, scan for it and connect if found
          ble.startScan([], function(device) {
            if (device['name'] == 'X9') {
              if (verbose)
                console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
              if (device['id'] == MAC) { // X9 found...
                ble.connect(MAC, function(device) {
                  if (verbose)
                    console.log('cordova-plugin-swt-x9: connected to X9');
                  if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
                    CONNECTED_X9_MAC = MAC;
                  }

                  if (verbose)
                    console.log('cordova-plugin-swt-x9: listening for sleep data notification response (3)');
                  ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
                    if (verbose)
                      console.log('cordova-plugin-swt-x9: sleep data notification response (3)', response);
                    var responseUintArray = new Uint8Array(response);
                    var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
                    resolve(responseUintArray, responseStr);
                  }, function(error) {
                    if (verbose)
                      console.error('cordova-plugin-swt-x9: sleep notification error (3)');
                    reject(error);
                  });
                }, function(error) {
                  if (verbose)
                    console.error('cordova-plugin-swt-x9: connection error (3)');
                  reject(error);
                });
              }
            }
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: scan error', error);
            scanning = false;
            reject(error);
          });

          // TODO: timeout
        }
      });
    }
  });
}

exports.getVerbosity = function() {
  return verbose;
}

exports.toggleVerbosity = function() { // toggle verbose console logging mode
  verbose = !verbose;
  if (verbose)
    console.log('cordova-plugin-swt-x9: toggled debug mode, set to', verbose);
}
